@@ -1,7 +1,7 @@
 /* Composite sequence support.
    Copyright (C) 2001, 2002, 2003, 2004, 2005,
-                 2006, 2007 Free Software Foundation, Inc.
-   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
+                 2006, 2007, 2008 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
      National Institute of Advanced Industrial Science and Technology (AIST)
      Registration Number H14PRO021
 
@@ -9,7 +9,7 @@ This file is part of GNU Emacs.
 
 GNU Emacs is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
+the Free Software Foundation; either version 3, or (at your option)
 any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
@@ -500,7 +500,8 @@ update_compositions (from, to, check_mask)
 	 avoid it, in such a case, we change the property of the
 	 latter to the copy of it.  */
       if (from > BEGV
-	  && find_composition (from - 1, -1, &start, &end, &prop, Qnil))
+	  && find_composition (from - 1, -1, &start, &end, &prop, Qnil)
+	  && COMPOSITION_VALID_P (start, end, prop))
 	{
 	  if (from < end)
 	    Fput_text_property (make_number (from), make_number (end),
@@ -510,7 +511,8 @@ update_compositions (from, to, check_mask)
 	  from = end;
 	}
       else if (from < ZV
-	       && find_composition (from, -1, &start, &from, &prop, Qnil))
+	       && find_composition (from, -1, &start, &from, &prop, Qnil)
+	       && COMPOSITION_VALID_P (start, from, prop))
 	run_composition_function (start, from, prop);
     }
 
@@ -521,6 +523,7 @@ update_compositions (from, to, check_mask)
          (to - 1).  */
       while (from < to - 1
 	     && find_composition (from, to, &start, &from, &prop, Qnil)
+	     && COMPOSITION_VALID_P (start, from, prop)
 	     && from < to - 1)
 	run_composition_function (start, from, prop);
     }
@@ -528,7 +531,8 @@ update_compositions (from, to, check_mask)
   if (check_mask & CHECK_TAIL)
     {
       if (from < to
-	  && find_composition (to - 1, -1, &start, &end, &prop, Qnil))
+	  && find_composition (to - 1, -1, &start, &end, &prop, Qnil)
+	  && COMPOSITION_VALID_P (start, end, prop))
 	{
 	  /* TO should be also at composition boundary.  But,
 	     insertion or deletion will make two compositions adjacent
@@ -542,7 +546,8 @@ update_compositions (from, to, check_mask)
 	  run_composition_function (start, end, prop);
 	}
       else if (to < ZV
-	       && find_composition (to, -1, &start, &end, &prop, Qnil))
+	       && find_composition (to, -1, &start, &end, &prop, Qnil)
+	       && COMPOSITION_VALID_P (start, end, prop))
 	run_composition_function (start, end, prop);
     }
 }
