@@ -1,12 +1,13 @@
 /* Lock files for editing.
    Copyright (C) 1985, 1986, 1987, 1993, 1994, 1996, 1998, 1999, 2000, 2001,
-                 2002, 2003, 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+                 2002, 2003, 2004, 2005, 2006, 2007, 2008
+                 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
 GNU Emacs is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
+the Free Software Foundation; either version 3, or (at your option)
 any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
@@ -468,8 +469,8 @@ current_lock_owner (owner, lfname)
     }
 
   /* Parse USER@HOST.PID:BOOT_TIME.  If can't parse, return -1.  */
-  /* The USER is everything before the first @.  */
-  at = index (lfinfo, '@');
+  /* The USER is everything before the last @.  */
+  at = rindex (lfinfo, '@');
   dot = rindex (lfinfo, '.');
   if (!at || !dot)
     {
