@@ -1,11 +1,12 @@
 /* m/ file for IBM 370 running AIX.
-   Copyright (C) 1993, 1994, 2001 Free Software Foundation, Inc.
+   Copyright (C) 1993, 1994, 2001, 2002, 2003, 2004,
+                 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
 GNU Emacs is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
+the Free Software Foundation; either version 3, or (at your option)
 any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
@@ -15,8 +16,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+Boston, MA 02110-1301, USA.  */
 
 #include "ibmps2-aix.h"
 
@@ -29,7 +30,7 @@ Boston, MA 02111-1307, USA.  */
 #undef INTEL386
 #undef aix386
 
-/* Define WORDS_BIG_ENDIAN iff lowest-numbered byte in a word
+/* Define WORDS_BIG_ENDIAN if lowest-numbered byte in a word
    is the most significant byte.  */
 
 #define WORDS_BIG_ENDIAN
@@ -49,8 +50,10 @@ Boston, MA 02111-1307, USA.  */
 #define LOAD_AVE_TYPE double
 
 #undef LIBS_MACHINE
-#define LIBS_MACHINE 
-#undef HAVE_VFORK
+#define LIBS_MACHINE
 
 #undef LD_SWITCH_MACHINE
 #define LD_SWITCH_MACHINE -xa
+
+/* arch-tag: 8605b600-0580-4e49-9ba9-8b4a977f860a
+   (do not change this comment) */
