@@ -1,11 +1,11 @@
 /* Copyright (C) 1985, 1986, 1987, 1988, 1992, 1993, 1994, 2001, 2002, 2003,
-                 2004, 2005, 2006, 2007  Free Software Foundation, Inc.
+                 2004, 2005, 2006, 2007, 2008  Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
 GNU Emacs is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
+the Free Software Foundation; either version 3, or (at your option)
 any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
@@ -693,7 +693,7 @@ make_hdr (new, a_out, data_start, bss_start, entry_address, a_name, new_name)
 	}
       if (coffheader.f_magic != COFF_MAGIC)
 	{
-	  ERROR1("%s doesn't have legal coff magic number\n", a_name);
+	  ERROR1("%s doesn't have valid coff magic number\n", a_name);
 	}
 #endif
       if (read (a_out, &ohdr, sizeof hdr) != sizeof hdr)
