@@ -1,11 +1,12 @@
 /* Heap management routines for GNU Emacs on the Microsoft W32 API.
-   Copyright (C) 1994 Free Software Foundation, Inc.
+   Copyright (C) 1994, 2001, 2002, 2003, 2004, 2005,
+                 2006, 2007, 2008  Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
 GNU Emacs is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
+the Free Software Foundation; either version 3, or (at your option)
 any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
@@ -15,13 +16,15 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.
+the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+Boston, MA 02110-1301, USA.
 
    Geoff Voelker (voelker@cs.washington.edu)			     7-29-94
 */
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -29,7 +32,6 @@ Boston, MA 02111-1307, USA.
 #include "w32heap.h"
 #include "lisp.h"  /* for VALMASK */
 
-#undef RVA_TO_PTR
 #define RVA_TO_PTR(rva) ((unsigned char *)((DWORD)(rva) + (DWORD)GetModuleHandle (NULL)))
 
 /* This gives us the page size and the size of the allocation unit on NT.  */
@@ -40,10 +42,6 @@ OSVERSIONINFO osinfo_cache;
 
 unsigned long syspage_mask = 0;
 
-/* These are defined to get Emacs to compile, but are not used.  */
-int edata;
-int etext;
-
 /* The major and minor versions of NT.  */
 int w32_major_version;
 int w32_minor_version;
@@ -56,9 +54,9 @@ int os_subtype;
 void
 cache_system_info (void)
 {
-  union 
+  union
     {
-      struct info 
+      struct info
 	{
 	  char  major;
 	  char  minor;
@@ -155,11 +153,11 @@ sbrk (unsigned long increment)
 {
   void *result;
   long size = (long) increment;
-  
+
   result = data_region_end;
-  
+
   /* If size is negative, shrink the heap by decommitting pages.  */
-  if (size < 0) 
+  if (size < 0)
     {
       int new_size;
       unsigned char *new_data_region_end;
@@ -170,7 +168,7 @@ sbrk (unsigned long increment)
       if ((data_region_end - size) < data_region_base)
 	return NULL;
 
-      /* We can only decommit full pages, so allow for 
+      /* We can only decommit full pages, so allow for
 	 partial deallocation [cga].  */
       new_data_region_end = (data_region_end - size);
       new_data_region_end = (unsigned char *)
@@ -186,9 +184,9 @@ sbrk (unsigned long increment)
  	}
 
       data_region_end -= size;
-    } 
+    }
   /* If size is positive, grow the heap by committing reserved pages.  */
-  else if (size > 0) 
+  else if (size > 0)
     {
       /* Sanity checks.  */
       if ((data_region_end + size) >
@@ -207,7 +205,7 @@ sbrk (unsigned long increment)
       real_data_region_end = (unsigned char *)
 	  ((long) (data_region_end + syspage_mask) & ~syspage_mask);
     }
-  
+
   return result;
 }
 
@@ -230,7 +228,7 @@ init_heap ()
   PIMAGE_NT_HEADERS nt_header;
 
   dos_header = (PIMAGE_DOS_HEADER) RVA_TO_PTR (0);
-  nt_header = (PIMAGE_NT_HEADERS) (((unsigned long) dos_header) + 
+  nt_header = (PIMAGE_NT_HEADERS) (((unsigned long) dos_header) +
 				   dos_header->e_lfanew);
   preload_heap_section = find_section ("EMHEAP", nt_header);
 
@@ -243,14 +241,15 @@ init_heap ()
 	  exit (1);
 	}
 
+#if defined (NO_UNION_TYPE) && !defined (USE_LSB_TAG)
       /* Ensure that the addresses don't use the upper tag bits since
 	 the Lisp type goes there.  */
-      if (((unsigned long) data_region_base & ~VALMASK) != 0) 
+      if (((unsigned long) data_region_base & ~VALMASK) != 0)
 	{
 	  printf ("Error: The heap was allocated in upper memory.\n");
 	  exit (1);
 	}
-
+#endif
       data_region_end = data_region_base;
       real_data_region_end = data_region_end;
     }
@@ -272,11 +271,11 @@ round_heap (unsigned long align)
 {
   unsigned long needs_to_be;
   unsigned long need_to_alloc;
-  
+
   needs_to_be = (unsigned long) ROUND_UP (get_heap_end (), align);
   need_to_alloc = needs_to_be - (unsigned long) get_heap_end ();
-  
-  if (need_to_alloc) 
+
+  if (need_to_alloc)
     sbrk (need_to_alloc);
 }
 
@@ -302,3 +301,6 @@ _heap_term (void)
 }
 
 #endif
+
+/* arch-tag: 9a6a9860-040d-422d-8905-450dd535cd9c
+   (do not change this comment) */
